use rodio::{source::SineWave, Sink};
use std::sync::atomic::{AtomicU8, Ordering};
use std::thread;
use std::time::{Duration, Instant};

static SOUND_TIMER: AtomicU8 = AtomicU8::new(0);
static DELAY_TIMER: AtomicU8 = AtomicU8::new(0);

pub enum Type {
    Delay,
    Sound,
}

pub struct Timer;

impl Timer {
    pub fn start_thread() {
        thread::spawn(|| {
            let duration = Duration::from_nanos(1e+9 as u64 / 60);
            let mut start = Instant::now();
            let mut has_beeped = false;
            loop {
                if Instant::now().duration_since(start) > duration {
                    Timer::tick(&mut has_beeped);
                    start = Instant::now();
                }
            }
        });
    }

    fn tick(has_beeped: &mut bool) {
        // Sound Timer
        let remaining = SOUND_TIMER.load(Ordering::Relaxed);
        match (remaining, *has_beeped) {
            (0, false) => {
                Self::beep();
                *has_beeped = true;
            }
            (0, true) => {}
            _ => {
                if *has_beeped {
                    *has_beeped = false;
                }

                SOUND_TIMER.store(remaining - 1, Ordering::Release)
            }
        }

        // Delay Timer
        let remaining = DELAY_TIMER.load(Ordering::Relaxed);
        if remaining != 0 {
            DELAY_TIMER.store(remaining - 1, Ordering::Release);
        }
    }

    pub fn get(timer: Type) -> u8 {
        match timer {
            Type::Delay => DELAY_TIMER.load(Ordering::Acquire),
            Type::Sound => SOUND_TIMER.load(Ordering::Acquire),
        }
    }

    pub fn set(secs: u8, timer: Type) {
        match timer {
            Type::Delay => DELAY_TIMER.store(secs, Ordering::Relaxed),
            Type::Sound => SOUND_TIMER.store(secs, Ordering::Relaxed),
        }
    }

    fn beep() {
        let maybe_device = rodio::default_output_device();
        let beep = SineWave::new(440);

        // TODO: Fix Pop when sound ends?
        if let Some(device) = maybe_device {
            std::thread::spawn(move || {
                let sink = Sink::new(&device);
                let duration = Duration::from_millis(100);
                let start = Instant::now();
                sink.append(beep);

                loop {
                    if Instant::now().duration_since(start) > duration {
                        break;
                    }
                }
            });
        }
    }
}
